dnl $Id: aclocal.m4,v 1.3 2000/10/10 22:10:43 dps Exp $
dnl some checks of my own

dnl A few simple macros

dnl Check a pgp signed file.
AC_DEFUN(cps_pgp_check,
[
AC_MSG_CHECKING([PGP signature of $2])
sigtest="untested"
case "$1" in
pgpv) sigtest=`pgpv -f <$2 2>&1 | ${cps_grep} "BAD signature"` ;;
pgp) sigtest=`pgp files.md5 -o /dev/null 2>&1 | ${cps_grep} "Bad signature"` ;;
esac
if test "${sigtest}" = "untested"; then
  AC_MSG_RESULT(untested)
else if test -n "${sigtest}"; then
  AC_MSG_RESULT([corrupt])
  AC_MSG_ERROR(Abort due to bad signature)
else
  AC_MSG_RESULT(OK)
fi; fi])

dnl check md5 signatures
AC_DEFUN(cps_md5_check,
[AC_REQUIRE([AC_PROG_AWK])
if test "$1" != "not found"; then
AC_MSG_CHECKING([md5 sums of files])
files=`$1 -c $2 2>/dev/null |
      $AWK 'BEGIN { f=""; FS=":"; n=0; }
      /:\ FAILED$/ { if (n==0) { f=$[1]; } else { f=f ", " $[1]; } n=1; }
      END { print f; }'`
if test -n "${files}"; then
  AC_MSG_RESULT(${files} corrupt)
  AC_MSG_ERROR([Abort due to corrupt files])
else
  AC_MSG_RESULT(OK)
fi; fi])

dnl find /proc
AC_DEFUN(cps_check_procfs,
[AC_CACHE_CHECK([proc filing system path], cps_cv_proc_path,
[AC_PROVIDE([$0])
cps_cv_proc_path=""
MY_PID=$$
for xcps_proc_path in $1; do
  if test -z "${cps_proc_path}"; then
    if test -d ${xcps_proc_path}; then
      if test -f "${xcps_proc_path}/${MY_PID}"; then
	 cps_cv_proc_path="${xcps_proc_path}"
      fi
      if test -d "${xcps_proc_path}/${MY_PID}"; then
	 cps_cv_proc_path="${xcps_proc_path}"
      fi
    fi
  fi
done
if test -z "${cps_cv_proc_path}"; then
  $cps_cv_proc_path=$2;
fi])])

dnl find ps database
AC_DEFUN(cps_check_psdb,
[AC_CACHE_CHECK([ps database filing system path], cps_cv_ps_db_path,
[AC_PROVIDE([$0])
cps_cv_ps_db_path=""
for xcps_ps_db_path in $1; do
  if test -z "${cps_ps_db_path}"; then
    if test -r ${xcps_ps_db_path}; then
      cps_cv_ps_db_path="${xcps_ps_bv_path}"
    fi
  fi
done
if test -z "${cps_cv_ps_db_path}"; then
  $cps_cv_ps_db_path=$2;
fi])])


dnl check type of processes
AC_DEFUN(cps_process_type,
[AC_REQUIRE([cps_check_procfs])
AC_MSG_CHECKING(type of a processes in $1)
AC_CACHE_VAL(cps_cv_proc_pidt,
[if test -d $1/$$; then
  cps_cv_proc_pidt="directory"
elif test -f $1/$$; then
  cps_cv_proc_pidt="regular file"
elif test -c $1/$$; then
  cps_cv_proc_pidt="character device"
elif test -b $1/$$; then
  cps_cv_proc_pidt="block device"
elif test -p $1/$$; then
  cps_cv_proc_pidt="fifo"
elif test -S $1/$$; then
  cps_cv_proc_pidt="socket"
elif test -L $1/$$; then
  cps_cv_proc_pidt="symbolic link"
elif test -t $1/$$; then
  cps_cv_proc_pidt="terminal"
else
  cps_cv_proc_pidt="unsupported type"
fi])
AC_MSG_RESULT($cps_cv_proc_pidt)
case "$cps_cv_proc_pidt" in
directory) AC_DEFINE($2, S_IFDIR) ;;
"regular file") AC_DEFINE($2, S_IFREG) ;;
"character device") AC_DEFINE($2, S_IFCHR) ;;
*) AC_MSG_ERROR([Type $cps_cv_proc_pidt not supported]) ;;
esac])


dnl setuid and setgid testing
AC_DEFUN(cps_setuid,
[AC_MSG_CHECKING([whether $1 is setuid])
AC_CACHE_VAL(cps_cv_$2_setuid,
[if test -u $1; then
   cps_cv_$2_setuid="yes"
else
   cps_cv_$2_setuid="no"
fi])
AC_MSG_RESULT(${cps_cv_$2_setuid})
if test "${cps_cv_$2_setuid}" = "yes"; then
  $3
else
  $4
fi
])

AC_DEFUN(cps_setgid,
[AC_MSG_CHECKING([whether $1 is setgid])
AC_CACHE_VAL(cps_cv_$2_setgid,
[if test -g $1; then
   cps_cv_$2_setgid="yes"
else
   cps_cv_$2_setgid="no"
fi])
AC_MSG_RESULT(${cps_cv_$2_setgid})	
if test "${cps_cv_$2_setgid}" = "yes"; then
  $3
else
  $4
fi])

dnl A moderately evil macro for figuring out the right arguments
dnl to pass to ps.
AC_DEFUN(cps_check_ps_args,
[AC_REQUIRE([AC_PROG_AWK])
AC_MSG_CHECKING([arguments to feed $1])
AC_CACHE_VAL(cps_cv_$2_args,
[cps_cv_$2_args=""
MY_PID=$$
for args in $3; do
   if test -z "${cps_cv_$2_args}"; then
     rm -fr conftest_args conftest_psout
     $1 $args >conftest_psout 2>conftest_args
     psstat=$?
     if test -f conftest_psout; then
       #Ensure process 1 and me are listed
       allp=`$AWK 'BEGIN { s=0; }
	     { if ([$]1==1 || [$]1=='${MY_PID}') { s++; }}
             END { if (s==2) { print 1; } else { print 0; }}' <conftest_psout`
     else
       allp=0
     fi
     if test $psstat -eq 0 && test $allp -eq 1; then 
       if test -f conftest_args; then
         cps_cv_$2_args=${args}
       else
         if test -z conftest_args; then
           cps_cv_$2_args=${args}
      	 fi
       fi
     fi
   fi
done
rm -fr conftest_args conftest_psout
if test -z "${cps_cv_$2_args}"; then
  AC_MSG_ERROR([I need to know what arguments to pass $1])
fi])
AC_MSG_RESULT(${cps_cv_$2_args})
])


dnl Find a user or group (merged version of two functions)
AC_DEFUN(cps_get_id,
[AC_REQUIRE([AC_PROG_AWK])
var=cps_cv_$2_$4
AC_MSG_CHECKING([$4 of $2])
AC_CACHE_VAL($var,
[eval $var=\"\"
if test -r /etc/passwd; then
   id=`$AWK 'BEGIN { FS=":"; }
	{ if ([$]1=="'$2'" && NF==$5) { print [$]3; } }' <$6`
   if test -n "$id"; then
     eval $var'='$id
   fi
fi])
if test -r /etc/passwd; then
  if eval 'test -z "[$]{'$var'}"'; then
    $1=$3
    AC_MSG_RESULT([no $7 $2, using uid $3])
    AC_MSG_WARN([Please add $7 $2, to your $6 file and reconfigure me])
  else
    eval '$1=${'${var}'}'
    AC_MSG_RESULT([$]$1)
  fi
else
  $1=$3
  AC_MSG_RESULT([no $6 file, using uid $3])
  AC_MSG_WARN([No $6 file on your system])
fi])


AC_DEFUN(cps_get_uid,
[cps_get_id($1,$2,$3,uid,7,/etc/passwd,user)])

AC_DEFUN(cps_get_gid,
[cps_get_id($1,$2,$3,gid,4,/etc/group,group)])


dnl Require some headers to exist
AC_DEFUN(cps_require_headers,
[AC_CHECK_HEADERS($1,,AC_MSG_ERROR([Required header $ac_hdr not found]))])

AC_DEFUN(cps_require_tool,
[define(tool_var, translit($1, 'a-z', 'A-Z'))
AC_CHECK_TOOL(tool_var,  $1, :)
if test "$tool_var" == ":"; then
	AC_MSG_ERROR([Required program $1 not found])
fi
AC_SUBST(tool_var, tool_var)
])

dnl Require some functions to exist
AC_DEFUN(cps_require_funcs,
[for func in $1; do
  got="no"
  tr_func=HAVE_`echo $func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
  AC_CHECK_FUNC($func, [AC_DEFINE_UNQUOTED($tr_func,1) got="yes"])
  for lib in $2; do
    if test "$got" = "no"; then
      AC_CHECK_LIB($lib, $func, [AC_DEFINE_UNQUOTED($tr_func,1)
	LIBS="$LIBS $lib"; got="yes"])
    fi
  done
  if test "$got" = "no"; then
    AC_MSG_ERROR([Required function $func not supported])	
  fi
done])

dnl check open supports all the symbols we need
AC_DEFUN(cps_check_open,
[AC_MSG_CHECKING([open supports my requirements])
AC_CACHE_VAL(cps_cv_open_works,
[AC_TRY_LINK(
[#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif],
[int a,b;

a=open("foo", O_CREAT | O_EXCL | O_WRONLY, 0700);
b=open("bar", O_RDONLY);],
cps_cv_open_works="yes", cps_cv_open_works="no")])
AC_MSG_RESULT("${cps_cv_open_works}")
if test "${cps_cv_open_works}" = "no"; then
   AC_MSG_ERROR([Your system can not cope with my use of open(2)])
fi])

dnl Check snprintf for overrun potential
AC_DEFUN([cps_snprintf_oflow],
[AC_MSG_CHECKING(whether snprintf ignores n)
AC_CACHE_VAL(cps_cv_snprintf_bug,
[AC_TRY_RUN(
changequote(<<, >>)dnl
<<#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */

#ifndef HAVE_SNPRINTF
#ifdef HAVE_VSNPRINTF
#include "vsnprintf.h"
#else /* not HAVE_VSNPRINTF */
#include "vsnprintf.c"
#endif /* HAVE_VSNPRINTF */
#endif /* HAVE_SNPRINTF */

int main(void)
{
char ovbuf[7];
int i,r;
FILE *f;
if ((f=fopen("conftest.d", "w"))==NULL)
  exit(1);
r=1;
for (i=0; i<7; i++) ovbuf[i]='x';
snprintf(ovbuf, 5,"foo%s", "bar");
if (ovbuf[5]!='x') exit(2);
if (ovbuf[4]!=0) r=1;
snprintf(ovbuf, 5,"foo%d", 666);
if (ovbuf[5]!='x') exit(2);
if (ovbuf[4]!=0) r=1;
fputc(r+'0', f);
fputc('\n', f);
fclose(f);
exit(0);
} >> 
changequote([, ]), cps_cv_snprintf_bug=`cat conftest.d`, cps_cv_snprintf_bug=2,
cps_cv_snprintf_bug=3)])
if test $cps_cv_snprintf_bug -eq 0; then
  AC_MSG_RESULT([no, snprintf is ok and it terminates])
else if test $cps_cv_snprintf_bug -eq 1; then
  AC_MSG_RESULT([no, snprintf is ok])
  AC_DEFINE(HAVE_SNPRINTF_UNTERM, 1)
else if test $cps_cv_snprintf_bug -eq 2; then
  AC_MSG_RESULT([yes, snprintf is broken])
  AC_DEFINE(HAVE_SNPRINTF_BUG,1)
  AC_MSG_WARN(Due to buggy snprintf it might be possible to buffer overrun me)
else
  AC_MSG_RESULT([unknown, assuming yes])
  AC_DEFINE(HAVE_SNPRINTF_BUG,1)
fi; fi; fi])

dnl Check vsnprintf for overrun potential
AC_DEFUN([cps_vsnprintf_oflow],
[AC_MSG_CHECKING(whether vsnprintf ignores n)
AC_CACHE_VAL(cps_cv_vsnprintf_bug,
[AC_TRY_RUN(
changequote(<<, >>)dnl
<<#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif /* HAVE_STDARG_H */

#ifndef HAVE_VSNPRINTF
#include "vsnprintf.c"
#endif /* HAVE_VSNPRINTF */

int prnt(char *s, const char *fmt, ...)
{
  va_list argp;
  va_start(argp, fmt);
  vsnprintf(s, 5, fmt, argp);
  va_end(argp);
}

int main(void)
{
  char ovbuf[7];
  int i,r;
  FILE *f;
  if ((f=fopen("conftest.d", "w"))==NULL)
	exit(1);
  r=0;
  for (i=0; i<7; i++) ovbuf[i]='x';
  prnt(ovbuf, "foo%s", "bar");
  if (ovbuf[5]!='x') exit(2);
  if (ovbuf[4]!=0) r=1;
  prnt(ovbuf, "foo%d", 666);
  if (ovbuf[5]!='x') exit(2);
  if (ovbuf[4]!=0) r=1;
  fputc(r+'0', f);
  fputc('\n', f);
  fclose(f);
  exit(0);
} >>
changequote([, ]), cps_cv_vsnprintf_bug=`cat conftest.d`, cps_cv_vsnprintf_bug=2, cps_cv_vsnprintf_bug=3)])

if test $cps_cv_vsnprintf_bug -eq 0; then
  AC_MSG_RESULT([no, vsnprintcf is ok and it terminates])
else if test $cps_cv_vsnprintf_bug -eq 1; then
  AC_MSG_RESULT([no, vsnprintf is ok])
  AC_DEFINE(HAVE_VSNPRINTF_UNTERM, 1)
else if test $cps_cv_vsnprintf_bug -eq 2; then
  AC_MSG_RESULT([yes, vsnprintf is broken])
  AC_DEFINE(HAVE_VSNPRINTF_BUG,1)
  AC_MSG_WARN(Due to buggy snprintf it might be possible to buffer overrun me)
else
  AC_MSG_RESULT([unknown, assuming yes])
  AC_DEFINE(HAVE_VSNPRINTF_BUG,1)
fi; fi; fi])

dnl mkdir and symlink interaction bug test
AC_DEFUN(cps_symlink_mkdir_bug,
[AC_MSG_CHECKING(security of interaction between symlink and mkdir)
AC_CACHE_VAL(cps_cv_symlink_mkdir_bug,
[mkdir conftest.d
mkdir conftest.d/foo
AC_TRY_RUN(
changequote(<<, >>)dnl
<<#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#else
extern int errno;
#endif

int main(void)
{
  if (chdir("conftest.d")!=0)
    exit(1);
  if (symlink("foo/foo","bar")!=0)
    exit(1);
  if (mkdir("bar")==0 || errno!=EEXIST)
    exit(1);
  exit(0);
} >>
changequote([, ]), cps_cv_symlink_mkdir_bug=0,
[if test -d conftest.d/foo/foo; then
  cps_cv_symlink_mkdir_bug=2
else
  cps_cv_symlink_mkdir_bug=1
fi], cps_cv_symlink_mkdir_bug=3)
rm -rf conftest.d])
case "$cps_cv_symlink_mkdir_bug" in
0) AC_MSG_RESULT(secure) ;;
1) AC_MSG_RESULT(errno wrong but ok)
   AC_DEFINE(HAVE_SYMLINK_MKDIR_ERRNO_BUG) ;;
2) AC_MSG_RESULT(insecure)
   AC_DEFINE(HAVE_SYMLINK_MKDIR_SECURITY_HOLE)
   AC_DEFINE(HAVE_SYMLINK_MKDIR_ERRNO_BUG) ;;
3) AC_MSG_RESULT(assuming insecure)
   AC_DEFINE(HAVE_SYMLINK_MKDIR_SECURITY_HOLE)
   AC_DEFINE(HAVE_SYMLINK_MKDIR_ERRNO_BUG) ;;
*) AC_MSG_RESULT($cps_cv_symlink_mkdir_bug)
   AC_MSG_ERROR(Impossible value of cps_cv_symlink_mkdir_bug) ;;
esac])


dnl open and symlink interaction bug test
AC_DEFUN(cps_symlink_open_bug,
[AC_MSG_CHECKING(security of interaction between symlink and open)
AC_CACHE_VAL(cps_cv_symlink_open_bug,
[mkdir conftest.d
AC_TRY_RUN(
changequote(<<, >>)dnl
<<#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#else
extern int errno;
#endif

int main(void)
{
  int fd;
  if (chdir("conftest.d")!=0)
    exit(1);
  if (symlink("foo","bar")!=0)
    exit(1);
  if ((fd=open("bar", O_CREAT | O_EXCL | O_WRONLY, 0700))==0)
  {
        write(fd, "If the symlink was to .rhosts you would be unhappy", 50);
	close(fd);
	exit(1);
  }
  if (errno!=EEXIST)
    exit(1);
  exit(0);
} >>
changequote([, ]), cps_cv_symlink_open_bug=0,
[if test -r conftest.d/foo; then
  cps_cv_symlink_open_bug=2
else
  cps_cv_symlink_open_bug=1
fi], cps_cv_symlink_open_buf=3)
rm -rf conftest.d])
case "$cps_cv_symlink_open_bug" in
0) AC_MSG_RESULT(secure) ;;
1) AC_MSG_RESULT(errno wrong but ok)
   AC_DEFINE(HAVE_SYMLINK_OPEN_ERRNO_BUG) ;;
2) AC_MSG_RESULT(insecure)
   AC_DEFINE(HAVE_SYMLINK_OPEN_SECURITY_HOLE)
   AC_DEFINE(HAVE_SYMLINK_OPEN_ERRNO_BUG) ;;
3) AC_MSG_RESULT(assuming insecure)
   AC_DEFINE(HAVE_SYMLINK_OPEN_SECURITY_HOLE)
   AC_DEFINE(HAVE_SYMLINK_OPEN_ERRNO_BUG) ;;
*) AC_MSG_RESULT($cps_cv_symlink_open_bug)
   AC_MSG_ERROR(Impossible value of cps_cv_symlink_open_bug) ;;
esac])


dnl Check that symlinks can not bypass search perms (on some old
dnl versions of Linux they can... this is now fixed, of course).
AC_DEFUN(cps_symlink_perms_bypass,
[AC_REQUIRE([AC_PROG_LN_S])
mkdir conftest.d
mkdir conftest.d/foo
mkdir conftest.d/foo/bar
touch conftest.d/foo/baz
touch conftest.d/foo/bar/quux
$cps_chmod 600 conftest.d/foo
AC_MSG_CHECKING(whether symlinks can reach files in unsearchable directories)
ln -s foo/bar conftest.d/bar
if test -r conftest.d/bar; then
  AC_MSG_RESULT(yes)
  AC_DEFINE(HAVE_SYMLINK_PERMS_BYPASS_FILE_HOLE)
else
  AC_MSG_RESULT(no)
fi
AC_MSG_CHECKING(whether symlinks can reach files in subdirs of no search dirs)
ln -s  foo/baz/quux conftest.d/quux
if test -r conftest.d/quux; then
  AC_MSG_RESULT(yes)
  AC_DEFINE(HAVE_SYMLINK_PERMS_BYPASS_DIR_HOLE)
else
  AC_MSG_RESULT(no)
fi
$cps_chmod 700 conftest.d/foo
rm -rf conftest.d])

dnl Like AC_TRY_RUN but keeps output (and custom error message)
AC_DEFUN(cps_run_prog,
[if test "$cross_compiling" = "yes"; then
   AC_MSG_ERROR([Can not probe non-portable values when cross compiling])
fi
cat > conftest.$ac_ext <<EOF
[#]line __oline__ "configure"
#include "confdefs.h"
ifelse(AC_LANG, CPLUSPLUS, [#ifdef __cplusplus
extern "C" void exit(int);
#endif
])
[$1]
EOF
if AC_TRY_EVAL(ac_link) && test -s conftest && $2=`(./conftest 2>/dev/null)`
then
dnl Don't remove the temporary files here, so they can be examined.
ifelse([$3], , :, [$3])
else
echo "configure: failed program was:" >&AC_FD_CC
cat conftest.$ac_ext >&AC_FD_CC
ifelse([$4], , , [  rm -fr conftest*
  $4
])
fi
rm -fr conftest* ])

dnl Find out device number for /dev/null
AC_DEFUN([cps_device_num],
[AC_MSG_CHECKING([device number for $1])
define(dev_nam, patsubst($1, [^/]))
define(dev_var, translit([cps_cv_]dev_nam, '/', '_'))
define(dev_val, translit(dev_nam[_RDEV], 'a-z/', 'A-Z_'))
AC_CACHE_VAL(dev_var,
[cps_run_prog([#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */

#ifndef Q
#define __Q(x) #x
#define Q(x) __Q(x)
#endif
int main(void)
{
struct stat stbuf;
if (stat(Q($1), &stbuf)!=0)
{
    if (errno==ENOENT)
	puts(Q($1) " not found");
    else
	puts("failed");
    exit(0);
}
printf("0x%04x", (int) stbuf.st_rdev);
exit(0);
}], dev_var,, dev_var="failed")])
AC_MSG_RESULT(${dev_var})
case "${dev_var}" in
0x*) AC_DEFINE_UNQUOTED(dev_val, ${dev_var});;
esac])

dnl Figure out the fs type number of the proc filing system
AC_DEFUN(cps_statfs_procfs_id,
[AC_REQUIRE([cps_check_procfs])
AC_MSG_CHECKING([for struct statfs])
AC_CACHE_VAL(cps_cv_have_struct_statfs,
AC_TRY_COMPILE([changequote(<<, >>)dnl
<<#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#endif >>
changequote([, ])], [struct statfs st; st.f_type=42],
cps_cv_have_struct_statfs="yes", cps_cv_have_struct_statfs="no"))
AC_MSG_RESULT($cps_cv_have_struct_statfs)
if test "$cps_cv_have_struct_statfs" = "no"; then
  $1=$2
  $4
else
AC_DEFINE(HAVE_STRUCT_STATFS)
AC_MSG_CHECKING([fs id of proc filing system])
AC_CACHE_VAL(cps_cv_proc_fstype,
[cps_run_prog(
changequote(<<, >>)dnl
<<#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#endif
#ifndef Q
#define __Q(x) #x
#define Q(x) __Q(x)
#endif
int main(void)
{
struct statfs fsbuf;
long ft;
if (statfs("/", &fsbuf)!=0)
  exit(1);
ft=fsbuf.f_type;
if (statfs(Q(${cps_cv_proc_path}) "/1", &fsbuf)!=0)
  exit(1);
if (ft!=fsbuf.f_type)
  printf("0x%08lx", fsbuf.f_type);
else
  puts("statfs useless");
exit(0);
} >>
changequote([, ]), cps_cv_proc_fstype,, cps_cv_proc_fstype="a fatal error occured")])
AC_MSG_RESULT($cps_cv_proc_fstype)
if test "${cps_cv_proc_fstype}" = "a fatal error occured"; then
  $1=$2
  $4
else if test "${cps_cv_proc_fstype}" = "statfs useless"; then
  $1=$2
  $4
else
  $1=$cps_cv_proc_fstype
  $3
fi; fi; fi ])

dnl Figure out the fs type number of the proc filing system
AC_DEFUN(cps_pid_max_probe,
[AC_MSG_CHECKING([whether I can compute maximum pid])
AC_CACHE_VAL(cps_cv_max_pid,
[cps_run_prog(
[#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif /* HAVE_SIGNAL_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */
volatile int escape;

RETSIGTYPE alarm_catch(int sig)
{
    sig=sig;
    escape=1;
    alarm(0);
}

int main(void)
{
    pid_t pid, mpid;
    int i;
    int oflow;

    pid=2;
    for (oflow=1, pid=1; pid>0; pid <<=1)
    {
	mpid=pid;
	if (kill(pid, 0)==-1 && errno==EINVAL)
	{
	   oflow=0;
	   break;
	}
    }
    mpid+=(mpid-1);
    if (oflow && (mpid<0 || (mpid+1)>=0))
    {
	fputs("no", stdout);
	exit(0);
    }	
    signal(SIGALRM, alarm_catch); 
    alarm(1);
    for (pid=1, escape=0; pid<=mpid; pid++)
    {
	if (escape)
	    break;
    }
    alarm(0);
    if (pid!=mpid)
    {
	fputs("no", stdout);
        exit(0);
    }
    printf("0x%08lx", ((long) mpid)+1);
    exit(0);
} ], cps_cv_max_pid,,cps_cv_max_pid="a fatal error occured")])
AC_MSG_RESULT($cps_cv_max_pid)
if test "${cps_cv_max_pid}" = "a fatal error occured"; then
  $1=$2
  $4
else if test "${cps_cv_max_pid}" = "no"; then
  $1=$2
  $4
else
  $1=$cps_cv_max_pid
  $3
fi; fi ])

dnl Check to RLIMIT_NPROC resource limit
AC_DEFUN(cps_rlimit_nproc,
[AC_MSG_CHECKING(for working RLIMIT_NPROC resource limit)
AC_CACHE_VAL(cps_cv_rlimit_nproc,
[AC_TRY_RUN(
changequote(<<, >>)dnl
<<
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif /* HAVE_SIGNAL_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif /* HAVE_SYS_RESOURCE_H */

int main(void)
{
#ifdef RLIMIT_NPROC
    static const struct rlimit pid_lim={RLIMIT_NPROC, 1};
    pid_t f;

    signal(SIGCHLD, SIG_IGN);
    setrlimit(RLIMIT_NPROC, (struct rlimit *) &pid_lim);
    if ((f=fork())==0)
	exit(0);
    if (f==-1)
	exit(0); /* The fork() failed (the right thing) */
#endif
   exit(1);
} >> 
changequote([, ]), cps_cv_rlimit_nproc=0, cps_cv_rlimit_nproc=1,
cps_cv_rlimit_nproc=2)])
if test $cps_cv_rlimit_nproc -eq 0; then
  AC_MSG_RESULT([yes])
  AC_DEFINE(HAVE_RLIMIT_NPROC,1)
else if test $cps_cv_rlimit_nproc -eq 1; then
  AC_MSG_RESULT([no])
else
  AC_MSG_RESULT([unknown, assuming none])
fi; fi])

dnl Check to RLIMIT_MEMLOCK resource limit
AC_DEFUN(cps_rlimit_memlock,
[AC_MSG_CHECKING(for RLIMIT_MEMLOCK resource limit)
AC_CACHE_VAL(cps_cv_rlimit_memlock,
[AC_TRY_RUN(
changequote(<<, >>)dnl
<<
#ifndef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifndef HAVE_SIGNAL_H
#include <signal.h>
#endif /* HAVE_SIGNAL_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif /* HAVE_SYS_RESOURCE_H */
#ifdef HAVE_SYS_MMAN
#include <sys/mman.h>
#endif /* HAVE_SYS_MMAN */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */

int main(void)
{
#ifdef RLIMIT_MEMLOCK
    static const struct rlimit mlock_lim={RLIMIT_MEMLOCK, 0};
    void *memory;

    if (setrlimit(RLIMIT_MEMLOCK, (struct rlimit *) &mlock_lim)!=-1)
	exit(0);
#endif
exit(1);
} >> 
changequote([, ]), cps_cv_rlimit_memlock=0, cps_cv_rlimit_memlock=1,
cps_cv_rlimit_memlock=2)])
if test $cps_cv_rlimit_memlock -eq 0; then
  AC_MSG_RESULT([yes])
  AC_DEFINE(HAVE_RLIMIT_MEMLOCK,1)
else if test $cps_cv_rlimit_memlock -eq 1; then
  AC_MSG_RESULT([no])
else
  AC_MSG_RESULT([unknown, assuming none])
fi; fi])

