/* $Id: ipread.c,v 1.2 2000/10/10 22:10:44 dps Exp $ */
/* Read an IP address */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */
#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif /* HAVE_CTYPE_H */
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif /* HAVE_SYS_SOCKET_H */
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif /* HAVE_NETINET_IN_H */

#include "detail.h"

int read_ip(FILE *in, struct sockaddr_in *res)
{
    int n, ns, c, portnum, digits;
    enum { N1=0, N2, N3, N4, PORT } state=N1;

    res->sin_family=AF_INET;
    res->sin_addr.s_addr=0;
    while ((c=fgetc(in))!=EOF)
    {
	switch(state)
	{
	case N2:
	case N3:
	case N4:
	    if (c!='.')
	    {
		ungetc(c, in);
		return 0;	/* Not an IP address */
	    }
	    if ((c=fgetc(in))==EOF)
	    {
		do_log("Unexpected EOF in octet of %d of IP adrdess", state+1);
		return 0;	/* Not an IP address */
	    }
	    /* FALL THRU */
	case N1:
	    ns=0;
	    if (!isdigit(c))
	    {
		ungetc(c, in);
		return 0;	/* Not a proper IP address */
	    }
	    if (c=='0')
	    {
		state++;	/* Move to next state */
		break;		/* 0 is the only allowed number statring with 0 */
	    }
		
	    /* If the code gets here c is a non-zero digit */
	    while(isdigit(c))
	    {
		ns=ns*10+c-'0';	/* >3 digits definately >255 */
		if (ns>255)
		    return 0;	/* Not a proper address */
		if ((c=fgetc(in))==EOF)
		{
		    do_log("Unexpected EOF in octet of %d of IP adrdess", state+1);
		    return 0;
		}
	    }
	    ungetc(c, in);	/* Push c back */
	    res->sin_addr.s_addr |= ns<<(state*8); /* or into address */
	    state++;
	    break;

	case PORT:
	    if (c!=':')
	    {
		ungetc(c, in);
		return 0;
	    }
	    if ((c=fgetc(in))==EOF)
	    {
		do_log("Unexpected EOF in octet of %d of IP adrdess", state+1);
		return 0;
	    }
	    if (c=='*')		/* Accept the likes of a.b.c.d:* */
	    {
		res->sin_port=0;
		return 1;
	    }
	    ungetc(c, in);	/* Push c back otherwise */
	    if ((portnum=readdec(in, &digits))!=-1)
	    {
		res->sin_port=(unsigned short int) (portnum & 65535);
		if (portnum & ~65535) /* Bits after the 16th test */
		{
		    do_log("Impossible port number %d (will store %d)", portnum,
			portnum & 65535);
		}
		return 1;	/* Everything OK */
	    }
	    return 0;		/* Not a proper address */
	}
    }
    do_log("Unexpected EOF in after of %d of IP adrdess", state+1);
    return 0;
}
