/* $Id: logsock.c,v 1.1 2000/05/22 18:27:06 dps Exp $ */
/* Socket detial logging */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif /* HAVE_SYS_SOCKET_H */
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif /* HAVE_NETINET_IN_H */
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif /* HAVE_ARPA_INET_H */
#ifdef HAVE_NETIPX_IPX_H
#include <netipx/ipx.h>
#endif /* HAVE_NETIPX_IPX_H */
#ifdef HAVE_SYS_UN_H
#include <sys/un.h>
#endif /* HAVE_SYS_UN_H */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif /* HAVE_FCNTL_H */
#ifdef HAVE_SYS_FCNTL_H
#include <sys/fcntl.h>
#endif /* HAVE_SYS_FCNTL_H */
#include "linux/os-cfg.h"
#include "libdetail/detail.h"
#include "prog.h"


static void print_sock(char *txt, int len, int status, const struct sockaddr *s, int l)
{
    const struct sockaddr_in *inet;
#ifdef HAVE_NETIPX_IPX_H
    const struct sockaddr_ipx *ipx;
#endif /* HAVE_NETIPX_IPX_H */
    const struct scokaddr_un *dom;
    int i, p;

    if (status!=0)
    {
	snprintf(txt, len, "[error %s]", strerror(errno));
	return;
    }
    switch(s->sa_family)
    {
    case AF_INET:
	inet=(const struct sockaddr_in *) s;
	snprintf(txt, len, "%s, port %d", inet_ntoa(inet->sin_addr),
		 ntohs(inet->sin_port));
	return;
	/* NOT REACHED */

#ifdef HAVE_NETIPX_IPX_H
    case AF_IPX:
	ipx=(const struct sockaddr_ipx *) s;
	snprintf(txt, len, "[ipx net %0hx, node %6s, port %0ux, type %d]",
		 ipx->sipx_network, ipx->sipx_node, ipx->sipx_port,
		 ipx->sipx_type);
	return;
	/* NOT REACHED */
#endif /* HAVE_NETIPX_IPX_H */

    case AF_UNIX:
	l-=offsetof(struct sockaddr, sa_data);
	snprintf(txt, len, "[socket ");
	for (i=0; i<l && i+9<len; i++)
	{
	    txt[i+8]=s->sa_data[i];
	}
	txt[i+8]='\0';
	return;
	/* NOT REACHED */

    default:
	break;
    }
    snprintf(txt, len, "[sock type %d", s->sa_family);
    l-=sizeof(s->sa_family);
    p=strlen(txt);
    for (i=0; i<l; i++, p+=3)
    {
	snprintf(txt+p, len-p, "%02x:", s->sa_data[i]);
    }
    return;
}


static void print_sockt(char *buf, int bufsiz, enum __socket_type t, int len)
{
    const char *s;

    /* First check the length */
    if (len!=sizeof(enum __socket_type))
    {
	snprintf(buf, bufsiz, "<type %d with wrong size %d (want %d)>",
		 (int) t, len, (int) sizeof(enum __socket_type));
	return;
    }
    switch(t)
    {
    case SOCK_STREAM:
	s="stream";
	break;

    case SOCK_DGRAM:
	s="datagram";
	break;

    case SOCK_RAW:
	s="raw";
	break;

    case SOCK_RDM:
	s="reliable message";
	break;

    case SOCK_SEQPACKET:
	s="reliable datagram";
	break;

    case SOCK_PACKET:
	s="wiretap packet (linux, obselete)";
	break;

    default:
	s="unknown";
	break;
    }
    snprintf(buf, bufsiz, "%s (%d)", s, (int) t);
}


void log_socket(int fd, int pid, const struct stat *stbuf)
{
    struct sockaddr peer, local;
    char localtxt[SOCKBUF], peertxt[SOCKBUF], typetxt[SOCKBUF];
    int locst, locallen, peerst, peerlen;
    enum __socket_type sock_type;
    socklen_t type_len;

    peerlen=sizeof(peer);
    peerst=getpeername(fd, &peer, &peerlen);
    print_sock(peertxt, sizeof(peertxt), peerst, &peer, peerlen);
    locallen=sizeof(local);
    locst=getsockname(fd, &local, &locallen);
    print_sock(localtxt, sizeof(localtxt), locst, &local, locallen);
    if (getsockopt(fd, SOL_SOCKET, SO_TYPE, &sock_type, &locallen)==0)
    {
	print_sockt(typetxt, sizeof(typetxt), sock_type, locallen);
    }
    else
    {
	/* getsockopt refused to tell us */
	snprintf(typetxt, sizeof(typetxt), "<getsock failed with %s>",
		 strerror(errno));
    }
    do_log("pid %d: fd %d is a %s socket from %s to %s (owner u=%d g=%d)", pid, fd,
	typetxt, peertxt, localtxt, (int) stbuf->st_uid, (int) stbuf->st_gid);
}    
