/* $Id: pidlist_proc.c,v 1.1.1.1 2000/05/04 22:56:29 dps Exp $ */
/* Read process names from /proc */

#include "config.h"
#include <stdio.h>
#include <ctype.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* #ifdef HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif /* HAVE_ERRNO_H */
#include "cfg_check.h"
#include "prog.h"
#include "numbuf.h"

/* The headers required for readdir() support. Lots of options here! */
#ifdef HAVE_DIRENT_H
#include <dirent.h>
#else
#define dirent direct
#ifdef HAVE_SYS_NDIR_H
#include <sys/ndir.h>
#elif defined(HAVE_SYS_DIR_H)
#include <sys/dir.h>
#elif defined(HAVE_NDIR_H)
#include <ndir.h>
#else
#error readdir not supported on your system.
#endif
#endif /* HAVE_DIRENT_H */

/* fs type support */
#ifdef STATFS_WORKS
#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#endif /* HAVE_SYS_VFS_H */
#endif /* STATFS_WORKS */

/* lstat hacks for broken systems without it */
#ifndef HAVE_LSTAT
#define lstat(x,y) stat(x,y)
#endif /* HAVE_LSTAT */

/* is number tester */
static int is_num(const char *s)
{
    if (*s=='\0')
       return 0;
    while (isdigit(*s)) s++;
    if (*s!='\0')
       return 0;
    return 1;
}

/* Pid tester. Anti-race measure. Also used for extra paranioa in pid_list */
int current_process(int pid)
{
    char pathbuf[PROCFNAME_MAX];	/* Lots */
    struct stat stbuf;
#if defined(HAVE_STATFS) && defined(CPS_PROC_FS_TYPE) && defined(STATFS_WORKS)
    struct statfs fts;
#endif
    
    if ((snprintf(pathbuf, sizeof(pathbuf), quote(PROCPATH) "/%d",
		  pid))>=sizeof(pathbuf)-1)
    {
	/* I think this is impossible but paranioa is vital here */
	do_log("Pathbuf overflow pid=%d", pid);
	return 1;			/* Assume process is active */
    }
#if defined(HAVE_STATFS) && defined(CPS_PROC_FS_TYPE) && defined(STATFS_WORKS)
    if (statfs(pathbuf, &fts)!=0)
    {
	if (errno==ENOENT)
	   return 0;
	do_log("Unexpected failure of statfs (error %d)", errno);
	return 1;
    }
    if (fts.f_type!=CPS_PROC_FS_TYPE)
    {
	do_log("Uh. Uh. Detected hacking attempt: wrong fs type!");
	return 1;
    }
#endif /* HAVE_STATFS and STATFS_FST and STATFS_WORKS */
    if (lstat(pathbuf, &stbuf)!=0)
    {
	if (errno==ENOENT)
	    return 0;		/* Expected error */
	do_log("Unexpected errno from lstat: %d", errno); /* Log errno */
	return 1;			/* Assume process is active */
    }
    else
    {
	if ((stbuf.st_mode & S_IFMT)!=CPS_PROC_PIDFMT)
	    do_log("Hacked libc or /proc. %s wrong type", pathbuf);
	return 1;		/* Process is active */
    }
    /* Not reached */
    do_log("Serious bug. Code got to current_process IP1");
    return 1;
}

/* build list from numbers listed in /proc */
numbuf *pid_list(void)
{
    DIR *dp;
    struct dirent *filep;
    int probed_pid;
    numbuf *r;

    if ((dp=opendir(quote(PROCPATH)))==NULL)
    {
       do_log(quote(PROCPATH));
       return NULL;
    }

    if ((r=allocate())==NULL)
    {
	do_log("Could not allocate space for numbers");
	return NULL;
    }

    /* Make sure all pids are listed */
    while ((filep=readdir(dp))!=NULL)
    {
	if (is_num(filep->d_name))
	{
	    probed_pid=atoi(filep->d_name);
	    if (!current_process(probed_pid))
	      continue;		/* Mainly for paranioa checks */
 
	    if (r->pos==r->max && expand_buf(r)==0)
	    {
		free_nums(r);
		return NULL;
	    }
	    r->nums[(r->pos)++]=probed_pid;
	}
    }
    return r;
}
