/* $Id: spoof.c,v 1.1.1.1 2000/05/04 22:54:51 dps Exp $ */

/* Enhanced spoofing copies the executable into a protected dir in /tmp
 * under the spoofed name and excutes in there. The daemon then removes all
 * this... added because /proc in linux records the real leaf name and
 * execvp does not chnage this. Deleting the executable causes no problems.
 * Where will the program stop? The code is paranoid here too, of course.
 */

/* Directory name to use, chosen to be fun to type... most systems
 * should handle it. \004 is ^D, i.e. EOF on terminals. \032 is ^Z, the mess
 * dos version of ^D. \0x7f is delete. \0x97 is not ISTRIP typable.
 */
#include "config.h"
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif /* HAVE_FCNTL_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#include "cfg_prog.h"
#include "prog.h"

#ifndef DEBUG
/* This filename is harder that you think to delete.... */
static const char *const dname=
	quote(TMPDIR) "/. \n\004\032!\r??\004\n\032%c%c%c!!  \004\032";
#else
/* This filename is easy */
static const char *const dname=
	quote(TMPDIR) "/cps%c%c%c";
#endif
#define MAXPATH 100
#define SIZE_INC 50


/* Deallaocate values */
static void free_vals(char **vals)
{
    char **scan=vals;

    while(*scan!=NULL)
    {
	free((void *) *scan);
	scan++;
    }
    free(vals);
}


/* enhanced spoofing cleabnup */
void enhanced_spoof_cleanup(const char *name, char **vals)
{
    FILE *in;
    int res;

    if (vals==NULL)
    {
	if ((in=fopen(quote(OPTFILE),"r"))==NULL)
	{
	    do_log("cleanup: Could not read values\n");
	    return;
	}
	if ((vals=readargs(in))==NULL)
	{
	    do_log("cleanup: Could not read values\n");
	    fclose(in);
	    return;
	}
	fclose(in);
    }
    /* Here vals!=NULL */
    unlink(name);
    unlink(quote(OPTFILE));
#ifdef DEBUG
    fprintf(stderr, "Removed `%s' and `%s'\n", name, quote(OPTFILE)); 
#endif
    chdir("..");
    rmdir(vals[0]);

#ifdef DEBUG
    fprintf(stderr, "Deleted directory `%s'\n", vals[0]);
#endif
    free_vals(vals);
}


/* cleanup function */
static void cleanup(const char *name)
{
    char **vals;
    FILE *in;
   
    if ((in=fopen(quote(OPTFILE),"r"))==NULL)
    {
	fprintf(stderr, "cleanup: Could not read values\n");
	return;
    }
    if ((vals=readargs(in))==NULL)
    {
	fprintf(stderr, "cleanup: Could not read values\n");
	fclose(in);
	return;
    }
    fclose(in);
    unlink(name);
    unlink(quote(OPTFILE));
    chdir("..");
    unlink(vals[0]);
    free_vals(vals);
}


/* Search path and open filename */
FILE *pathsrch(const char *const progname)
{
    const char *path, *pscan;
    char *t;
    char filename[MAXPATH];
    FILE *f;
    int i;
    struct stat stbuf;

    if ((path=(char *)getenv("PATH"))==NULL || *path=='\0')
	return fopen(progname, "r"); /* PATH undefined or empty */

    if (strchr(progname,'/')!=NULL)
	return fopen(progname, "r"); /* Path name given */

    for (pscan=path, f=NULL; f==NULL && *pscan!='\0'; )
    {
	/* Copy path element */
	for (i=0, t=filename; *pscan!='\0' && *pscan!=':' && i<MAXPATH; i++)
	    *(t++)=*(pscan++);
	if (*t!='/')
	    *t='/';
	if (i>0)
	{
	    t++;
	    i++;
	}
	if (*pscan)
	   pscan++;		/* Skip : */
	if (i+strlen(progname)>=MAXPATH)
	    continue;
	strcpy(t, progname);
#ifdef DEBUG
	fprintf(stderr, "Testing `%s'\n", filename);
#endif
	if ((f=fopen(filename,"r"))==NULL)
	    continue;
	if (fstat(fileno(f), &stbuf))
	{
	    do_log("fstat failed for file %s", t);
	    fclose(f);
	    continue;
	}
	if (!S_ISREG(stbuf.st_mode))
	{
	    fclose(f);
	    continue;
	}
    }

    /* Ouch! Nasty trailing colon handling */
    if (f==NULL && *(pscan-1)==':')	/* Trailing : ? */
    {
	if ((f=fopen(progname, "r"))!=NULL)
	{
	    if (fstat(fileno(f), &stbuf))
	    {
		do_log("fstat failed for file %s", t);
		fclose(f);
	    }
	    if (!S_ISREG(stbuf.st_mode))
	    {
		fclose(f);
	    }
	}
    }
    return f;
}

/* Make directory. Hopefully symlink brain damage does not apply in your
 * operating systems (I am not aware of any with this but someone might)
 */
int makedirs(int argc, char *argv[])
{
    char dirname[MAXPATH];
    char p,q,r;
    int i, fd;
    FILE *s;
#ifdef HAVE_SYMLINK_MKDIR_SECURITY_HOLE
    struct stat stbuf;
#endif /* HAVE_SYMLINK_MKDIR_SECURITY_HOLE */

    p=q=r='0';
    while(1)
    {
	sprintf(dirname, dname, p, q, r);
#ifdef HAVE_SYMLINK_MKDIR_SECURITY_HOLE
	/* Race-able but better than nothing */
	if (lstat(dirname, &stbuf)=-1 && errno!=ENOENT)
	{
#endif
	    i=mkdir(dirname, 0700); /* Mode 0700 is the right thing[TM] */
#ifdef HAVE_SYMLINK_MKDIR_SECURITY_HOLE
	}
#endif
	if (i==0) {
            chown(dirname, DAEMON_UID, DAEMON_GID);
	    break;
        }
	if (++r<='Z')
	    continue;
	r='0';
	if (++q<='Z')
	    continue;
	q='0';
	if (++p<='Z')
	    continue;
	fprintf(stderr, "Hack attack! All my filenames failed (fatal)\n");
	exit(1);
    }
    /* FIXME: Hard to exploit race here. */
    if (chdir(dirname)!=0)
    {
	fprintf(stderr, "Hack attack! chdir() failed (fatal)\n");
	exit(1);
    }
#ifdef HAVE_SYMLINK_OPEN_SECURITY_HOLE
    /* Race-able but better than nothing */
    if (lstat(quote(OPTFILE), &stbuf)=-1 && errno!=ENOENT &&
	(fd=open(quote(OPTFILE), O_EXCL | O_CREAT | O_WRONLY,
		 S_IRUSR | S_IWUSR))==-1)
#else
    /* Sane systems are ok without lstat checks are have not races */
    if ((fd=open(quote(OPTFILE), O_EXCL | O_CREAT | O_WRONLY,
		S_IRUSR | S_IWUSR))==-1)
#endif
    {
	fprintf(stderr, "Hack attack! fopen failed (fatal)\n");
	chdir("..");
	rmdir(dirname);
	exit(1);
    }

    if ((s=fdopen(fd, "w"))==NULL)
    {
	fprintf(stderr, "Hack attack! fdopen failed (fatal)\n");
	chdir("..");
	rmdir(dirname);
	exit(1);
    }

    fprintf(s,"%d", argc);
    fputc('\0', s);
#ifdef DEBUG
    fprintf(stderr, "Write: args=%d\n", argc);
#endif
    for (i=0; i<argc; i++)
    {
	fputs(argv[i], s);
	fputc('\0', s);
#ifdef DEBUG
	fprintf(stderr, "Write: arg[%d]=`%s'\n", i, argv[i]);
#endif
    }
    fputs(dirname, s);
#ifdef DEBUG
    fprintf(stderr, "Write: dir=`%s'\n", dirname);
#endif
    fputc('\0',s);
    fclose(s);
    return 1;
}


/* File copy */
int copy_file(const char *name, FILE *in)
{
    FILE *out;
    char buf[BUFSIZ];
    size_t rd_sz, wr_sz, nwr;
    int fd;
#ifdef HAVE_SYMLINK_OPEN_SECURITY_HOLE
    struct stat stbuf;
 
    /* Raceable but better than nothing */
    if (lstat(name, &stbuf)==-1 && errno==ENOENT &&
	(fd=open(name, O_EXCL | O_CREAT | O_WRONLY,
		S_ISUID | S_IXUSR | S_IXGRP | S_IXOTH))==-1)
#else
    /* Sane systems are ok without lstat checks are have not races */
    if ((fd=open(name, O_EXCL | O_CREAT | O_WRONLY,
		S_ISUID | S_IXUSR | S_IXGRP | S_IXOTH))==-1)
#endif
    {
	fclose(in);
	cleanup(name);
	return 0;
    }

    if ((out=fdopen(fd, "w"))==NULL)
    {
        close(fd);
	fprintf(stderr, "Could not open output file\n");
	fclose(in);
	cleanup(name);
	return 0;
    }

    while ((rd_sz=fread((void *) buf, sizeof(char), sizeof(buf), in))>0)
    {
	wr_sz=0;
	while (wr_sz<rd_sz)
	{
	    nwr=fwrite((void *) buf, sizeof(char), rd_sz-wr_sz, out);
	    if (nwr==0)
	    {
		fprintf(stderr, "Could not write output file\n");
		fclose(out);
		cleanup(name);
		return 0;
	    }
	    wr_sz+=nwr;
	}
    }
#ifdef HAVE_FCHMOD
    fchmod(fileno(out), S_ISUID | S_IXUSR | S_IXGRP | S_IXOTH ); /* Set mode */
#endif /* HAVE_FCHMOD */
    fclose(out);
    fclose(in);
#ifndef HAVE_FCHMOD
    chmod(name, S_ISUID | S_IXUSR | S_IXGRP | S_IXOTH ); /* Set mode */
#endif /* not HAVE_FCHMOD */
    return 1;
}


/* Read zero terminated string */
static char *zstring(FILE *in)
{
    char *line;
    int c, i, size;

    if ((line=(char *)malloc(SIZE_INC))==NULL)
	return NULL;
    i=0; size=SIZE_INC;
    while (c=fgetc(in))
    {
	if (i==size)
	{
	    size+=SIZE_INC;
	    if ((line=(char *)realloc(line, size))==NULL)
		return NULL;
	}
	if (c=='\0' || c==EOF)
	    break;
	line[i]=c;
	i++;
    }
    line[i]='\0';

    return strdup(line);
}

/* Free values */
static void free_values(char **vals, int idx)
{
    int i;

    for (i=1; i<idx; i++)
	free((void *) vals[i]);
    free((void *) vals);
}

/* Read arguments */
char **readargs(FILE *in)
{
    char **vals;
    char *line;
    int nums, idx;

    /* Read argumnet count */
    if ((line=zstring(in))==NULL)
	return NULL;
    nums=atoi(line);
    free((void *) line);
    if (nums<0)
	return NULL;
#ifdef DEBUG
    fprintf(stderr, "readargs: %d args\n", nums);
#endif

    /* Allocate space */
    if ((vals=(char **)malloc((nums+2)*sizeof(const char **)))==NULL)
	return NULL;

    /* Reads args */
    idx=1;
    while (nums-->0)
    {
	if ((line=zstring(in))==NULL)
	{
	    free_values(vals, idx);
	    return NULL;
	}
	vals[idx]=line;
#ifdef DEBUG
	fprintf(stderr, "readargs: arg[%d] is `%s'\n", idx, line);
#endif
	idx++;
    }
    vals[idx]=NULL;

    /* Read directory name. This is utter untypable for extra security. */
    line=zstring(in);
#ifdef DEBUG
    fprintf(stderr, "Directory `%s'\n", line);
#endif
    if (line==NULL)
    {
	free_values(vals, idx);
	vals=NULL;
    }
    else
	vals[0]=line;		/* idx[0] is dirname */
    return vals;		/* Return values */
}

int count_vals(char **p)
{
    int n=0;

    while (*(p+n)!=NULL)
	n++;
    return n;
}
